%% Multivariate Probit
% 
%
%  written by: 
%      Aline Talhouk
%      Department of Statistics           
%      University of British Columbia
%      http://www.stat.ubc.ca/~a.talhouk
%
%% 
%  Last modified: August 18, 2011
%
%  Disclaimer:  No guarantee that this code is free of bugs.
%               If you find any mistakes please email a.talhouk@stat.ubc.ca
%%

function [Csamples iCsamples Bsamples]= mvprobit(y,X,N,sat,fname)
%Multivariate Probit regression with Binary data

% Input:
% y binary n x J response variable
% X is n x p design matrix enter a zeros matrix if there are no covariates
% N number of Gibbs samples to be collected
% sat indicator of whether to use a saturated model or perform covariance
%    selection

% Output:
% Csamples: Correlation samples
% iCsamples: Inverse Correlation samples
% Bsamples: Regression coefficient samples
%
% Aline Talhouk, UBC (2011)

[n,J]=size(y);
p=size(X,2);

if sum(sum(X))==0 % if there are no covariates set the regression coefficient parameters to 0
    covar=0;
    iXi=zeros(p,p);
    M=zeros(p,J);
    Xbhat=zeros(n,J);
else
    covar=1;
    iP=eye(p)*0.00000000001; %Prior on Betas
    iXi=X'*X+iP;
    Xi=inv(iXi);
end


kep=1;

Sig=eye(J);
Rinv=eye(J);
z0=zeros(n,J);
ind=[];


%Start timer
itime=cputime;

for s = 1:N
    %Parameter Expansion
    %Sample D from the prior
    id2=gamrnd((J+1)/2,2./diag(Rinv),J,1);
    D = diag(1./sqrt(id2));
    
    w=z0*D;

    %Sample gamma if covariates
    if covar
    [gamma M] = sampleBETA(Sig,w,X,Xi);
    betahat=gamma*Dinv;
    Xbhat=X*betahat;
    end
   
    %Sample R
    if sat
        [Rhat Rinv Sig Dinv]=sampleR(w,iXi,M);
    else
        %Propose graph
        G=sampleG(w,iXi,M);
        [Rhat Rinv Sig Dinv] = constrR(G,w,iXi,M);
    end

     %Sample Latent
     for l=1:n
         z0(l,:)=rmvtnorm(Xbhat(l,:)',Rinv,z0(l,:)',y(l,:)',[-Inf 0 Inf])';
     end
    
    

    
    %Save samples
    Csamples(s,:) = triuEl(Rhat)';
    iCsamples(s,:)=triuEl(Rinv)';
    if covar
    Bsamples(s,:) = reshape(betahat,1,J*p);
    else 
        Bsamples=NaN;
    end

    %Timer
    if(s==100)||(s==kep*1000)
        %disp('We are at iteration')
        %disp(s);
        save(fname);
        kep=kep+1;
        ctime=cputime;
        timetoend=((ctime - itime)/s) * (N + 1 - s);
        disp('Minutes remaining')
        disp(timetoend/60);
        ttime=(ctime-itime)/60;
    end

end

end
